@@ -8,3 +8,7 @@ class TerminalException(Exception):
 
 class NoFrameRateException(Exception):
     """ An exception raised due to frame rate not found."""
+
+
+class TranscriptionException(Exception):
+    """ An exception raised due to transcription failures."""
