@@ -4,7 +4,7 @@ usage: subaligner_batch [-h] [-m {single,dual}] [-vd VIDEO_DIRECTORY] [-sd SUBTI
                         [-sil {afr,amh,ara,arg,asm,aze,ben,bos,bul,cat,ces,cmn,cym,dan,deu,ell,eng,epo,est,eus,fas,fin,fra,gla,gle,glg,grc,grn,guj,heb,hin,hrv,hun,hye,ina,ind,isl,ita,jbo,jpn,kal,kan,kat,kir,kor,kur,lat,lav,lfn,lit,mal,mar,mkd,mlt,msa,mya,nah,nep,nld,nor,ori,orm,pan,pap,pol,por,ron,rus,sin,slk,slv,spa,sqi,srp,swa,swe,tam,tat,tel,tha,tsn,tur,ukr,urd,vie,yue,zho}]
                         [-fos] [-tod TRAINING_OUTPUT_DIRECTORY] [-od OUTPUT_DIRECTORY] [-t TRANSLATE] [-lgs] [-d] [-q] [-ver]
 
-Batch align multiple subtitle files and audiovisual files (v0.1.4)
+Batch align multiple subtitle files and audiovisual files
 
 Subtitle files and their companion audiovisual files need to be stored in two separate directories.
 Each file pair needs to share the same base filename, the part before the extension.
@@ -96,6 +96,7 @@ Each file pair needs to share the same base filename, the part before the extens
         help="Switch on stretch on subtitles)",
     )
     from subaligner.utils import Utils
+    from subaligner.subtitle import Subtitle
     parser.add_argument(
         "-sil",
         "--stretch_in_language",
@@ -124,6 +125,14 @@ Each file pair needs to share the same base filename, the part before the extens
         default="",
         help="Path to the output subtitle directory",
     )
+    parser.add_argument(
+        "-of",
+        "--output_format",
+        type=str,
+        choices=list(map(lambda x: x[1:], Subtitle.subtitle_extensions())),
+        default="",
+        help="File format of the output subtitles"
+    )
     parser.add_argument(
         "-t",
         "--translate",
@@ -164,7 +173,7 @@ Each file pair needs to share the same base filename, the part before the extens
         sys.exit(21)
     if FLAGS.translate is not None:
         if "transformers" not in {pkg.key for pkg in pkg_resources.working_set}:
-            print('ERROR: Alignment has been configured to perform translation. Please install "subaligner[translation]" and run your command again.')
+            print('ERROR: Alignment has been configured to perform translation. Please install "subaligner[llm]" and run your command again.')
             sys.exit(21)
 
     video_file_paths = [os.path.abspath(os.path.join(path, p)) for path, _, files in
@@ -202,13 +211,13 @@ Each file pair needs to share the same base filename, the part before the extens
                 aligned_subs, audio_file_path, voice_probabilities, frame_rate = predictor.predict_single_pass(
                     video_file_path=local_video_path,
                     subtitle_file_path=local_subtitle_path,
-                    weights_dir=os.path.join(FLAGS.training_output_directory, "models/training/weights")
+                    weights_dir=os.path.join(FLAGS.training_output_directory, "models", "training", "weights")
                 )
             else:
                 aligned_subs, subs, voice_probabilities, frame_rate = predictor.predict_dual_pass(
                     video_file_path=local_video_path,
                     subtitle_file_path=local_subtitle_path,
-                    weights_dir=os.path.join(FLAGS.training_output_directory, "models/training/weights"),
+                    weights_dir=os.path.join(FLAGS.training_output_directory, "models", "training", "weights"),
                     stretch=stretch,
                     stretch_in_lang=stretch_in_lang,
                     exit_segfail=exit_segfail,
@@ -216,17 +225,18 @@ Each file pair needs to share the same base filename, the part before the extens
 
             parent_dir = os.path.dirname(local_subtitle_path.replace(os.path.abspath(FLAGS.subtitle_directory), output_dir))
             os.makedirs(parent_dir, exist_ok=True)
-            aligned_subtitle_path = os.path.abspath(os.path.join(parent_dir,
-                                                    ".".join(os.path.basename(local_subtitle_path).rsplit(".", 1)).replace(".stl", ".srt")))
+            file_parts = os.path.basename(local_subtitle_path).rsplit(".", 1)
+            file_parts[1] = FLAGS.output_format if FLAGS.output_format != "" else file_parts[1]
+            aligned_subtitle_path = os.path.abspath(os.path.join(parent_dir, ".".join(file_parts).replace(".stl", ".srt")))
 
             if FLAGS.translate is not None:
                 from subaligner.translator import Translator
                 source, target = FLAGS.translate.split(",")
                 translator = Translator(source, target)
                 aligned_subs = translator.translate(aligned_subs)
-                Subtitle.export_subtitle(local_subtitle_path, aligned_subs, aligned_subtitle_path, frame_rate, "utf-8")
+                Subtitle.save_subs_as_target_format(aligned_subs, local_subtitle_path, aligned_subtitle_path, frame_rate, "utf-8")
             else:
-                Subtitle.export_subtitle(local_subtitle_path, aligned_subs, aligned_subtitle_path, frame_rate)
+                Subtitle.save_subs_as_target_format(aligned_subs, local_subtitle_path, aligned_subtitle_path, frame_rate, "utf-8")
 
             log_loss = predictor.get_log_loss(voice_probabilities, aligned_subs)
             if log_loss is None or log_loss > FLAGS.max_logloss:
