@@ -59,6 +59,8 @@ class Subtitle(object):
 
         if subtitle_format == "subrip":
             self.__subs = self.__load_subrip(subtitle_file_path)
+        elif subtitle_format == "subrip_raw":
+            self.__subs = pysrt.SubRipFile().from_string(subtitle_file_path)
         elif subtitle_format == "ttml":
             self.__subs = self.__convert_ttml_to_subs(subtitle_file_path)
         elif subtitle_format == "webvtt":
@@ -105,6 +107,19 @@ class Subtitle(object):
 
         return cls(cls.__secret, subtitle_file_path, "subrip")
 
+    @classmethod
+    def load_subrip_str(cls, subrip_raw: str) -> "Subtitle":
+        """Load a SubRip subtitle string.
+
+        Arguments:
+            subrip_str {string} -- The string representation of the SubRip content.
+
+        Returns:
+            Subtitle -- Subtitle object.
+        """
+
+        return cls(cls.__secret, subrip_raw, "subrip_raw")
+
     @classmethod
     def load_ttml(cls, subtitle_file_path: str) -> "Subtitle":
         """Load a TTML subtitle file.
@@ -324,6 +339,10 @@ class Subtitle(object):
             string -- The path to the shifted subtitle file.
         """
         _, file_extension = os.path.splitext(subtitle_file_path)
+        if shifted_subtitle_file_path is None:
+            shifted_subtitle_file_path = subtitle_file_path.replace(
+                file_extension, "{}{}".format(suffix, file_extension)
+            )
         if file_extension.lower() in cls.TTML_EXTENSIONS:
             subs = cls(cls.__secret, subtitle_file_path, "ttml").subs
             subs.shift(seconds=seconds)
@@ -333,10 +352,6 @@ class Subtitle(object):
             for index, cue in enumerate(cues):
                 cue.attrib["begin"] = str(subs[index].start).replace(",", ".")
                 cue.attrib["end"] = str(subs[index].end).replace(",", ".")
-            if shifted_subtitle_file_path is None:
-                shifted_subtitle_file_path = subtitle_file_path.replace(
-                    file_extension, "{}{}".format(suffix, file_extension)
-                )
             encoding = Utils.detect_encoding(subtitle_file_path)
             tree.write(shifted_subtitle_file_path, encoding=encoding)
         elif file_extension.lower() in cls.STL_EXTENSIONS:
@@ -741,7 +756,7 @@ class Subtitle(object):
                 # Change single quotes in the XML header to double quotes
                 with open(target_file_path, "w", encoding=encoding) as target:
                     if "xml_declaration" in inspect.getfullargspec(ElementTree.tostring).kwonlyargs:  # for >= python 3.8
-                        encoded = ElementTree.tostring(tt, encoding=encoding, method="xml", xml_declaration=True)
+                        encoded = ElementTree.tostring(tt, encoding=encoding, method="xml", xml_declaration=True)  # type: ignore
                     else:
                         encoded = ElementTree.tostring(tt, encoding=encoding, method="xml")
                     normalised = encoded.decode(encoding) \
